;------------------------------------------------------------------------------
; Fu Kung! Copyright (C)2003 Andrew Davie
; see http://www.taswegian.com/TwoHeaded/Atari2600/fukung.html << SOON


; The genesis of the large-sprite system contained herein comes from Eckhard Stohlberg's
; fine work on the 'BIGTWO' sprite system (see the [stella] list archives at
; http://www.biglist.com/lists/stella/archives/ ) and in his preliminary fighter code
; kindly loaned to assist in creating this program.

; The sprite system this program uses is somewhat different to Eckhard's - being more
; efficient and capable (in terms of sprite display) but does essentially retain the
; positioning and movement system of the initial system.  Many thanks ES!

; This code is for a 32K cartridge using the $3F bank-switching scheme.  If running
; with Z26 emulator (v1.57+) use -g11 switch on command-line.


;------------------------------------------------------------------------------

                processor 6502
                include "vcs3f.h"

BGCHRONO        = 0

DEBUG           = 0         ; non-zero for debug stuff

BGCOL           = 0
PFCOL           = 0 ;%011100100 

D0              = 1
D1              = 2
D2              = 4
D3              = 8
D4              = 16
D5              = 32
D6              = 64
D7              = 128

BANK            = $3F       ; write address to switch banks
BANKPAGE        = $F000     ; base address of bank blocks

INITBANK        = 14
BANK_FRAMETABLE = 14
BANK_STATE      = 14
BANK_ANIMATE    = 14

;------------------------------------------------------------------------------
; Code automatically compiles to implement the sprites in the size/format
; specified here.  Can handle up to 6 rows.  

FRAMEBANK00     = 0
FRAMEBANK01     = 1
FRAMEBANK02     = 2
FRAMEBANK03     = 4
FRAMEBANK04     = 5
FRAMEBANK05     = 6
FRAMEBANK06     = 7
FRAMEBANK07     = 8
FRAMEBANK08     = 9
FRAMEBANK09     = 10
FRAMEBANK10     = 11
FRAMEBANK11     = 12
FRAMEBANK12     = 13

INCLUDEDBANKS   = 13        ; banks to actually compile - development only



BIG_ROWS        = 1
BIG_COLS        = 6         ; DO NOT CHANGE

BIG_SPRITES     = 2         ; #creatures

;------------------------------------------------------------------------------
; MACRO definitions 


    MAC VECTOR              ; just a word pointer to code
        .word {1}
    ENDM

    MAC SETBANK
            lda #{1}
            sta BANK
    ENDM

    MAC ANIMATION
            lda #<{1}
            sta ObjAnimLO,x
            lda #>{1}
            sta ObjAnimHI,x
            lda #1
            sta ObjDelay,x

			lda ObjFlag,x
			ora #OBJFLAG_WAIT			; animation in progress
			sta ObjFlag,x

    ENDM

    MAC HANDLER
            ldy #{1}*2
            jsr SetHandler
    ENDM

    MAC STATE       ; does not continue after macro!

            ldy #STATE_{1}*2
            jmp SetState

    ENDM


;------------------------------------------------------------------------------

                SEG.U variables
                ORG $80

DrawX           ds 2
DrawY           ds 1
CenterX         ds 2
CenterY         ds 1
FrameBank       ds 1

VisualX         ds 2
VisualY         ds 1

LoopCount       ds 1
SPBASE          ds BIG_ROWS * BIG_COLS * 2


BGColour        ds 1
BGPlayer        ds 1

Temp            ds 2
Anim            ds 2
RGBVec          ds 2

Player          ds 1
FrameAddress    ds 2
Phase           ds 1
Phase2          ds 1
switch          ds 1

;----
Flags           ds 1

                ; -------L
                ; L = locked to opponent

FLAG_LOCK       = D0

LineR0          ds 2                    ; 6 columns of RED sub-image
LineR1          ds 2
LineR2          ds 2
LineR3          ds 2
LineR4          ds 2
LineR5          ds 2

LineG0          ds 2                    ; 6 columns of GREEN sub-image
LineG1          ds 2
LineG2          ds 2
LineG3          ds 2
LineG4          ds 2
LineG5          ds 2

LineB0          ds 2                    ; 6 columns of BLUE sub-image
LineB1          ds 2
LineB2          ds 2
LineB3          ds 2
LineB4          ds 2
LineB5          ds 2


;----
; Object variables


ObjXPosition    ds BIG_SPRITES
ObjYPosition    ds BIG_SPRITES

ObjFacing         ds BIG_SPRITES

                ; F-------
                ; F = facing direction (0 = right, 1 = left)
                ; - = unused

ObjDelay        ds BIG_SPRITES              ; delay count for frame animation
ObjAnimLO       ds BIG_SPRITES              ; animation pointer lo byte
ObjAnimHI       ds BIG_SPRITES              ; animation pointer hi byte
objAnimBank     ds BIG_SPRITES              ; bank containing animation

    ; Note: Possibly include bank with animation hi byte

ObjStateLO      ds BIG_SPRITES              ; state table lo byte
ObjStateHI      ds BIG_SPRITES              ; state table hi byte

ObjFSMLO        ds BIG_SPRITES              ; FSM system lo byte
ObjFSMHI        ds BIG_SPRITES              ; FSM system hi byte

ObjFlag			ds BIG_SPRITES
ObjFrame        ds BIG_SPRITES

OBJFLAG_WAIT	= D7						; wait for animation


;------------------------------------------------------------------------------

        ; This overlay variable is used for the overlay variables.  That's OK.
        ; However, it is positioned at the END of the variables so, if on the off chance we're overlapping
        ; stack space and variable, it is LIKELY that that won't be a problem, as the temp variables
        ; (especially the latter ones) are only used in rare occasions.

        ; FOR SAFETY, DO NOT USE THIS AREA DIRECTLY (ie: NEVER reference 'Overlay' in the code)
        ; ADD AN OVERLAY FOR EACH ROUTINE'S USE, SO CLASHES CAN BE EASILY CHECKED

Overlay         ds 0 ;010       ;--> overlay (share) variables


;------------------------------------------------------------------------------
; OVERLAYS!
; These variables are overlays, and should be managed with care
; That is, variables are ALREADY DEFINED, and we're reusing RAM for other purposes

; EACH OF THESE ARE VARIABLES (TEMPORARY) USED BY ONE ROUTINE (AND IT'S SUBROUTINES)
; THAT IS, LOCAL VARIABLES.  USE 'EM FREELY, THEY COST NOTHING

; TOTAL SPACE USED BY ANY OVERLAY GROUP SHOULD BE <= SIZE OF 'Overlay'

;------------------------------------------------------------------------------

                org Overlay

    ; ANIMATION/LOGIC SYSTEM


;------------------------------------------------------------------------------

                org Overlay

    ; DRAWING SYSTEM


;------------------------------------------------------------------------------

            SEG
            ORG $0000
            RORG $f000

    ; GRAPHICS DATA LOADS HERE
    ; WE WILL HAVE MULTIPLE 2K BANKS, EACH USING AN INCLUDE FILE OF GENERATED DATA

            include "Graphics\bank00.asm"

    #if INCLUDEDBANKS > 1

            SEG
            ORG $0800
            RORG $f000
            include "Graphics\bank01.asm"
    #endif


    #if INCLUDEDBANKS > 2 

            SEG
            ORG $1000
            RORG $f000
            include "Graphics\bank02.asm"
    #endif
            
    ; << bank3 reserved for cuttle-compatibility


    #if INCLUDEDBANKS > 3

            SEG
            ORG $2000
            RORG $f000
            include "Graphics\bank03.asm"
    #endif

    #if INCLUDEDBANKS > 4
            SEG
            ORG $2800
            RORG $f000
            include "Graphics\bank04.asm"
    #endif

    #if INCLUDEDBANKS > 5
            SEG
            ORG $3000
            RORG $f000
            include "Graphics\bank05.asm"
    #endif

    #if INCLUDEDBANKS > 6
            SEG
            ORG $3800
            RORG $f000
            include "Graphics\bank06.asm"
    #endif

    #if INCLUDEDBANKS > 7
            SEG
            ORG $4000
            RORG $f000
            include "Graphics\bank07.asm"
    #endif

    #if INCLUDEDBANKS > 8 
            SEG
            ORG $4800
            RORG $f000
            include "Graphics\bank08.asm"
    #endif

    #if INCLUDEDBANKS > 9
            SEG
            ORG $5000
            RORG $f000
            include "Graphics\bank09.asm"
    #endif

    #if INCLUDEDBANKS > 10 
            SEG
            ORG $5800
            RORG $f000
            include "Graphics\bank10.asm"
    #endif

    #if INCLUDEDBANKS > 11
            SEG
            ORG $6000
            RORG $f000
            include "Graphics\bank11.asm"
    #endif

    #if INCLUDEDBANKS > 12 
            SEG
            ORG $6800
            RORG $f000
            include "Graphics\bank12.asm"
    #endif

;    SEG
;    ORG $6800
;    RORG $F000
;    #include "Graphics\bgchronored.asm" 
;    #include "Graphics\bgchronogreen.asm" 
;    #include "Graphics\bgchronoblue.asm" 
    
;------------------------------------------------------------------------------

            SEG
            ORG $7000           ; 2K, ie: bank 1
            RORG $f000          ; lives at $F000


InitX
.XP     SET 80-24


    REPEAT BIG_SPRITES
    .byte .XP
.XP     SET .XP + 48
    REPEND


    ; There's no particular reason why this code lives in a separate bank
    ; but it's as good-a-test as any for checking the bankswitching is working


Cart_Init
            sei
            cld
               
		    ldx #$FF
		    txs 				; Clear the stack

            lda #0
            sta SWBCNT          ; console I/O always set to INPUT
            sta SWACNT          ; set controller I/O to INPUT

GameInit

            lda #0
            sta Flags
            sta Player
            sta Phase
            ;sta Phase2

    ; Initialise creatures

            ldx #BIG_SPRITES-1
PreClear    

            lda #0
            sta ObjFacing,x
			sta ObjFlag,x

            lda #140
            sta ObjYPosition,x

            lda InitX,x
            sta ObjXPosition,x

            SETBANK BANK_STATE
            
            ldy #STATE_NORMAL*2
            jsr SetState                    ; also calls processFSM

#if BANK_STATE != BANK_ANIMATE
            SETBANK BANK_ANIMATE
#endif

            jsr AnimateObject               ; process object animation sequence

            dex
            bpl PreClear

    ;----------------------------------
    ; Hardwire init a single creature...

    #if BIG_SPRITES > 1

;            lda #D7
;            sta ObjFacing           ;todo currently fucked

            ldx #1
            HANDLER EVENT_INIT2

            ldx #0
            HANDLER 0

    #endif

;            ldx #0
;            HANDLER NORMAL_STAND
  

    ;----------------------------------


            lda #BGCOL
            sta BGColour

            jmp NewFrame



    include "Graphics\bank_FRAMETABLE.asm"


;------------------------------------------------------------------------------
; OBJECT Animation
; *MUST* BE IN ONE BANK


    MAC SHOW
            .byte {1},{2}
    ENDM

    MAC SHOWRGB
    REPEAT {2}
        SHOW FRAME_{1}RED,1
        SHOW FRAME_{1}GREEN,1
        SHOW FRAME_{1}BLUE,1
    REPEND
    ENDM

    MAC SYNCH
            .byte TOKEN_SYNCH
    ENDM
    
    MAC SETCOL
            .byte TOKEN_COLOUR
            .byte {1}
    ENDM

    MAC LOCK

            .byte TOKEN_LOCK
            .byte {1}*2

    ENDM


    MAC UNLOCK
            .byte TOKEN_UNLOCK
    ENDM

    MAC GOTO
            .byte TOKEN_JUMP
            .word {1}
    ENDM

	MAC DONE
			.byte TOKEN_DONE
	ENDM

    MAC MOVE
            .byte TOKEN_MOVE
            .word {1}
            .byte {2}
    ENDM

	MAC FLIP
			.byte TOKEN_FLIP
	ENDM



NOFRAME     equ $FF


noAnim      lda #NOFRAME
            sta ObjFrame,x
            rts


exitAnim    rts


AnimateObject


            lda ObjAnimHI,x
            beq noAnim                      ; animation switched off - use a null frame

            lda ObjDelay,x
            beq exitAnim                    ; static frame - no animation
            dec ObjDelay,x
            bne exitAnim                    ; not ready to change

ReAnimate   lda ObjAnimHI,x
            sta Anim+1
            lda ObjAnimLO,x
            sta Anim                        ; (Anim) points to frame sequence

ReReAnim    ldy #0
            lda (Anim),y
            bpl NormalFrame

            asl
            tay

            lda AnimVec,y
            sta Temp
            lda AnimVec+1,y
            sta Temp+1
            jmp (Temp)

    ;------------------------------------------------------------------------------
    ; Automatically define tokens and a vector table to token-processing code
    ; This is fairly neat, as the ordering of the entries in the table auto-generates
    ; the appropriate equates to access the table correctly!

TOK         SET 128
            MAC TOKEN
TOKEN_{1}   equ TOK
TOK         SET TOK + 1
            VECTOR Animate{1}
            ENDM

AnimVec     TOKEN JUMP
            TOKEN LOCK
            TOKEN MOVE
            TOKEN UNLOCK
            TOKEN DONE
            TOKEN FLIP
            TOKEN COLOUR
            TOKEN SYNCH


    ;------------------------------------------------------------------------------

AnimateDONE
			lda ObjFlag,x
			and #~OBJFLAG_WAIT
			sta ObjFlag,x
			
			lda #1
			jmp AnimUp



    ;------------------------------------------------------------------------------

AnimateFLIP

            lda ObjFacing,x
            eor #D7
            sta ObjFacing,x

			lda #1
			jmp AnimUp


NormalFrame

            sta ObjFrame,x

            iny
            lda (Anim),y
            sta ObjDelay,x                  ; should be < 128

            clc
            lda Anim
            adc #2
            sta ObjAnimLO,x
            lda Anim+1
            adc #0
            sta ObjAnimHI,x
            rts

    ;------------------------------------------------------------------------------

AnimateLOCK

                ; lock other creature to this

            lda ObjXPosition,x
            pha
            lda ObjYPosition,x
            pha

            lda Flags
            ora #FLAG_LOCK
            sta Flags

            txa
            eor #1
            tax                         ; other player

            ldy #1
            lda (Anim),y
            tay
            jsr SetHandler              ; cause other player to react

            pla
            sta ObjYPosition,x
            pla
            sta ObjXPosition,x

            txa
            eor #1
            tax

            lda #2

AnimUp      clc
            adc Anim
            sta Anim
            lda Anim+1
            adc #0
            sta Anim+1

            jmp ReReAnim

    ;------------------------------------------------------------------------------

AnimateUNLOCK

            lda Flags
            and #~FLAG_LOCK
            sta Flags

            lda #1
            jmp AnimUp


    ;------------------------------------------------------------------------------


AnimateJUMP

                ; Handle a JUMP

            ldy #1
            lda (Anim),y
            sta ObjAnimLO,x
            iny
            lda (Anim),y
            sta ObjAnimHI,x
            jmp ReAnimate


AnimateCOLOUR
            ldy #1
            lda (Anim),y
            sta BGPlayer
			lda #2
			jmp AnimUp

    ;------------------------------------------------------------------------------
            
AnimateMOVE

            ldy #1

            lda (Anim),y
            sta Temp
            iny
            lda (Anim),y
            sta Temp+1

            lda ObjFacing,x
            bpl MoveRight

            sec
            lda #0
            sbc Temp
            sta Temp
            lda #0
            sbc Temp+1
            sta Temp+1

MoveRight

            clc
            lda DrawX
            adc Temp
            sta DrawX
            lda DrawX+1
            adc Temp+1
            sta DrawX+1

            iny
            lda (Anim),y
            clc
            adc DrawY
            sta DrawY

            clc
            lda Anim
            adc #4
            sta Anim
            lda Anim+1
            adc #0
            sta Anim+1

            jmp ReReAnim

AnimateSYNCH

#if BIG_SPRITES = 1
            lda #0
#else
            lda #2
#endif

            sta Phase
            lda #1
            jmp AnimUp

    ;------------------------------------------------------------------------------


    ; Animations processing code proceeds until it has a frame and duration
    ; The duration is in TV-frames, and counts down.  When 0, the next 'command' in the
    ; animation sequence is processed.  Animations must use the GOTO to halt or repeat
    ; Animations are tokenised interpreted little programs.

    ; Commands/Macros
    ;   GOTO label              Vector to the label
    ;   LOCK event              Lock opponents together, sending the opponent the event
    ;   MOVE x,y                Adjust position by x,y pixels
    ;   UNLOCK                  Unlock opponents
    ;   DONE                    Indicate animation finished (clears OBJFLAG_WAIT for creature)
    ;   FLIP                    Mirror creature
    ;   SHOW frame,delay        Set th creature rame and delay (also halts animation till delay counts to 0)



ANIMATION_STAND


        SYNCH



ANIMATION_STAND2

;        SHOW FRAME_2RED,30

        SHOWRGB 2,2
        SHOWRGB 3,2
        SHOWRGB 6,2
        SHOWRGB 7,2
        SHOWRGB 8,2
        SHOWRGB 10,2
        SHOWRGB 11,2

        GOTO ANIMATION_STAND2


ANIMATION_RIGHT

        SHOWRGB 11,5
        GOTO ANIMATION_STAND2






        SHOWRGB 7,3
        SHOWRGB 8,3
        SHOWRGB 9,3
        SHOWRGB 10,3
        SHOWRGB 11,3
        SHOWRGB 12,3
        SHOWRGB 1,30
        SHOWRGB 2,3
        SHOWRGB 3,3
        SHOWRGB 4,3
        SHOWRGB 5,3


; OBJECT Animation END
;------------------------------------------------------------------------------
 

;------------------------------------------------------------------------------
; STATE Processing
; *MUST* BE IN ONE BANK

    MAC FACEOPPONENT

            txa
            eor #1
            tay
            lda ObjFacing,y
            sta ObjFacing,x
    
    ENDM

	MAC WAIT
	SUBROUTINE
.HERE
			lda #<.HERE
			sta ObjFSMLO,x
			lda #>.HERE
			sta ObjFSMHI,x

		; wait for animation to complete

			lda ObjFlag,x
			and #OBJFLAG_WAIT
			bne .CONT
			
		    lda #<{1}
            sta Temp
            lda #>{1}
            sta Temp+1
            jmp (Temp)						; vector to the handler now the wait has completed
.CONT
		; Or, wait is incomplete, so fall through

	ENDM



SetState

    ; Set object's state base
    ; x = obj
    ; y = state table (*2)

            lda StateTables,y
            sta ObjStateLO,x
            lda StateTables+1,y
            sta ObjStateHI,x

            ldy #EVENT_DEFAULT*2                ; fall through and include DEFAULT init for handler

SetHandler

    ; Change object's processing code
    ; x = obj
    ; y = new handler entry (word access)

            lda ObjStateHI,x
            beq NullHandler                     ; don't handle anything with no entry

            sta Temp+1
            lda ObjStateLO,x
            sta Temp

            iny
            lda (Temp),y
            beq NullHandler                     ; no handler present for given event
            sta ObjFSMHI,x
            dey
            lda (Temp),y
            sta ObjFSMLO,x

   

ProcessFSM

    ; Vector to the creature's processing code
    ; x = creature

            lda ObjFSMLO,x
            sta Temp
            lda ObjFSMHI,x
            sta Temp+1
            jmp (Temp)

NullHandler rts




    ;------------------------------------------------------------------------------
    ; List of events

EVENT_DEFAULT           = 0                     ; default entry when switching states
EVENT_JOY_NULL          = 1
EVENT_JOY_UP            = 2
EVENT_JOY_DOWN          = 3
EVENT_JOY_LEFT          = 4
EVENT_JOY_RIGHT         = 5
EVENT_ATTACK_THROW      = 6

EVENT_INIT2             = 2


    ;------------------------------------------------------------------------------
    ; Automatically define states and a vector table to state-processing code
    ; This is fairly neat, as the ordering of the entries in the table auto-generates
    ; the appropriate equates to access the table correctly!

STATEQ      SET 0
            MAC STATENTRY
STATE_{1}   equ STATEQ
STATEQ      SET STATEQ + 1
            VECTOR State{1}
            ENDM

StateTables STATENTRY NORMAL
            STATENTRY WAIT

    ;------------------------------------------------------------------------------
    ; NORMAL state table
    ; Each State table is a table of pointers to code processing 'EVENTS'

StateNORMAL
            VECTOR NormalInit                ; 0     EVENT_DEFAULT
            VECTOR NormalWait                ; 1     EVENT_JOY_NULL
            VECTOR NormalInitRight ;0 ;GetUp                     ; 4     EVENT_JOY_UP
            VECTOR 0 ;NormalInit;AttackThrow1              ; 5     EVENT_JOY_DOWN
            VECTOR 0;WalkTurnAround            ; 6     EVENT_JOY_LEFT
            VECTOR 0;WalkStart                 ; 7     EVEMT_JOY_RIGHT
            VECTOR 0;React_AttackThrow         ; 8     EVENT_ATTACK_THROW

    ;------------------------------------------------------------------------------


StateWAIT

        ; allows moves to complete without joystick interaction 
        ; TODO: should still react to other creature moves

			VECTOR BusyInit                  ; 0     EVENT_DEFAULT
            VECTOR 0                         ; 1     EVENT_JOY_NULL
            VECTOR 0                         ; 4     EVENT_JOY_UP
            VECTOR 0					     ; 5     EVENT_JOY_DOWN
            VECTOR 0						 ; 6     EVENT_JOY_LEFT
            VECTOR 0						 ; 7     EVEMT_JOY_RIGHT
            VECTOR 0                         ; 8     EVENT_ATTACK_THROW

    ;------------------------------------------------------------------------------
    ; BUSY
    ; Waits until the animation clears the wait flag
    ; Then reverts to STATE_NORMAL

BusyInit	WAIT BusyDone
			rts

BusyDone	STATE NORMAL


    ;------------------------------------------------------------------------------
    ; NORMAL
    ; The basic default standing, doing nothing.


NormalInit  ANIMATION ANIMATION_STAND
            HANDLER EVENT_JOY_NULL
            rts

NormalWait
            ;WAIT NormalInit
            rts


NormalInitRight

            ANIMATION ANIMATION_RIGHT
            HANDLER EVENT_JOY_NULL
            rts



; STATE Processing END
;------------------------------------------------------------------------------




;******************************************************************************



            SEG bank31
            ORG $7800
            RORG $f800

VBProcessing

      
            ldx Player

            lda ObjXPosition,x
            sta DrawX
            lda #1
            sta DrawX+1
            lda ObjYPosition,x
            sta DrawY

            jsr ProcessFSM                  ; process creature FSM logic

            SETBANK INITBANK

            jsr AnimateObject               ; process object animation sequence
            jsr CalculateDrawPosition       ; setup position(s) for next VB draw

    ;------------------------------------------------------------------------


            ldy Player

            lda ObjFacing,y
            and #D7
            lsr
            lsr
            lsr
            lsr
            sta REFP0
            sta REFP1                   ;@17

            lda BGColour
            sta COLUBK

_RTS14      nop
            rts                         ; 14-cycle waste

    ;------------------------------------------------------------------------

NewFrame

    ; Start of vertical blank processing

            lda #$02
            sta WSYNC

            sta VBLANK          

            sta VSYNC           
            sta WSYNC           
            sta WSYNC
            sta WSYNC

            lda #$00
            sta VSYNC           

            lda #43             
            sta TIM64T

            lda #$00
            sta HMCLR

            lda SWCHB
            sta switch


            jsr VBProcessing        ; spend 2000+ cycles

#if DEBUG
            lda INTIM
            bne Whoops
            inc BGColour            ; Overflow
Whoops
#endif



VblankLoop  lda INTIM
            bne VblankLoop      
            
            sta WSYNC           
            sta VBLANK          

    ;------------------------------------------------------------------------------
    ; START OF DISPLAY


            lda #238                ; NTSC
            sta TIM64T


    ;?
            lda     #%00010100      ; reflect, priority, 2 pixel wide ball
            sta     CTRLPF
    ;^?

            lda #PFCOL
            sta COLUPF
            
            lda #0
            sta PF0
            sta PF1
            sta PF2


            sta GRP0
            sta GRP1

            lda BGPlayer
            sta COLUP0
            sta COLUP1                  ; tentative colour


;            jsr DrawChrono


            lda FrameBank
            sta BANK


    ;--------------------------------------------------------------------------
    ; Init the loop counters (16 rows each) for each of the rows

            lda #127
            sta LoopCount

            ldy Player
            lda ObjFrame,y
            bmi SkipDraw

  

    ldx Phase

    lda Player
    bne NotUP

    lda NextPhase,x
    sta Phase
    tax

NotUP

    lda Player
    bne PalCols

    lda switch
    and #D7
    beq NTSC2

PalCols    txa
    clc
    adc #3
    tax

NTSC2

    txa
    asl
    tax
    lda PhasePtr,x
    sta RGBVec
    lda PhasePtr+1,x
    sta RGBVec+1

            jsr PositionObject
            jsr DrawSprite

            sta WSYNC
            
;            lda #RED
;            sta COLUBK

SkipDraw    lda #0
            sta GRP0
            sta GRP1
            sta GRP0            ; buffered??
            sta GRP1            ; buffered??

            sta VDELP0
            sta VDELP1

            sta REFP0
            sta REFP1



;-----------------


    ;--------------------------------------------------------------------------
    ; START OF COLOUR BITMAP DISPLAY KERNEL


;PadScreen1  lda INTIM
;            bne PadScreen1



;-------------------



;            lda #254
;            sta PF2

;            ldx #8
;x2          sta WSYNC
;            dex
;            bne x2

;            stx PF2
            sta WSYNC

PadScreen2  lda INTIM
            bne PadScreen2
  
            sta HMCLR
            sta WSYNC


    lda switch
    and #D7
    beq NTSC

    lda #60
    sta TIM64T
PAL lda INTIM
    bne PAL

NTSC


            lda #D6+D1
            sta VBLANK                      ; end of screen - enter blanking

OverscanStart

            lda #25 ;35             
            sta TIM64T


    ;--------------------------------------------------------------------------
    ; Switch to alternate player(s)
            
            dec Player
            bpl PlayerN
            lda #BIG_SPRITES-1
            sta Player
PlayerN


    ;--------------------------------------------------------------------------
    ; PLAYER INTERACTION


;            lda #0
;            sta SWACNT              ; all input bits (IMPLICITLY SET @ STARTUP)

;            lda SWCHA               ; joystick

;            ldx Player
;            bne P1shift

;            lsr
;            lsr
;            lsr
;            lsr

;P1shift     and #$f

;            ldy ObjFacing,x
;            bpl FacOK

;            tay
;            and #D3+D2
;            cmp #D3+D2
;            beq FacOK2

;            tya
;            eor #D3+D2                      ; swap left/right if mirrored
;FacOK       tay
;FacOK2

            SETBANK BANK_STATE

;            lda JoystickEvent,y
;           bmi NullJoy                     ; an unhandled joystick direction

;            asl
;            tay

            ldy #EVENT_JOY_NULL*2
            jsr SetHandler                  ; handle joystick event

;NullJoy
;                lda     SWCHB
;                and     #$01
;                bne     NoResetSwitch

Overscan    lda INTIM
            bne Overscan

            sta WSYNC           
            sta WSYNC           

            jmp NewFrame


#if BGCHRONO

DrawChrono

            lda #<bgchronored_STRIP_0
            sta LineR0
            lda #>bgchronored_STRIP_0
            sta LineR0+1

            lda #<bgchronored_STRIP_1
            sta LineR1
            lda #>bgchronored_STRIP_1
            sta LineR1+1

            lda #<bgchronored_STRIP_2
            sta LineR2
            lda #>bgchronored_STRIP_2
            sta LineR2+1

            lda #<bgchronored_STRIP_3
            sta LineR3
            lda #>bgchronored_STRIP_3
            sta LineR3+1

            lda #<bgchronored_STRIP_4
            sta LineR4
            lda #>bgchronored_STRIP_4
            sta LineR4+1

            lda #<bgchronored_STRIP_5
            sta LineR5
            lda #>bgchronored_STRIP_5
            sta LineR5+1



            lda #<bgchronogreen_STRIP_0
            sta LineG0
            lda #>bgchronogreen_STRIP_0
            sta LineG0+1

            lda #<bgchronogreen_STRIP_1
            sta LineG1
            lda #>bgchronogreen_STRIP_1
            sta LineG1+1

            lda #<bgchronogreen_STRIP_2
            sta LineG2
            lda #>bgchronogreen_STRIP_2
            sta LineG2+1

            lda #<bgchronogreen_STRIP_3
            sta LineG3
            lda #>bgchronogreen_STRIP_3
            sta LineG3+1

            lda #<bgchronogreen_STRIP_4
            sta LineG4
            lda #>bgchronogreen_STRIP_4
            sta LineG4+1

            lda #<bgchronogreen_STRIP_5
            sta LineG5
            lda #>bgchronogreen_STRIP_5
            sta LineG5+1


            lda #<bgchronoblue_STRIP_0
            sta LineB0
            lda #>bgchronoblue_STRIP_0
            sta LineB0+1

            lda #<bgchronoblue_STRIP_1
            sta LineB1
            lda #>bgchronoblue_STRIP_1
            sta LineB1+1

            lda #<bgchronoblue_STRIP_2
            sta LineB2
            lda #>bgchronoblue_STRIP_2
            sta LineB2+1

            lda #<bgchronoblue_STRIP_3
            sta LineB3
            lda #>bgchronoblue_STRIP_3
            sta LineB3+1

            lda #<bgchronoblue_STRIP_4
            sta LineB4
            lda #>bgchronoblue_STRIP_4
            sta LineB4+1

            lda #<bgchronoblue_STRIP_5
            sta LineB5
            lda #>bgchronoblue_STRIP_5
            sta LineB5+1

            SETBANK FRAMEBANK12
            
            ldy #49                ; total number of scanlines to display


    ; Vector to the correct start - a red, blue or green line


            ldx Phase2
            lda NextPhase,x
            sta Phase2

            sta WSYNC

            ldx Phase2
            beq ScanRed
            dex
            beq ScanGreen
            dex
            beq ScanBlue

whoops      bne whoops


ScanRed     sta WSYNC

            lda #RED
            sta COLUPF

            lda (LineR0),y
            sta PF0
            lda (LineR1),y
            sta PF1
            lda (LineR2),y
            sta PF2
            lda (LineR3),y
            sta PF0
            lda (LineR4),y
            sta PF1
            lda (LineR5),y
            sta PF2

            dey
            beq ScanEnd

          
ScanGreen   sta WSYNC

            lda #GREEN
            sta COLUPF

            lda (LineG0),y
            sta PF0
            lda (LineG1),y
            sta PF1
            lda (LineG2),y
            sta PF2
            lda (LineG3),y
            sta PF0
            lda (LineG4),y
            sta PF1
            lda (LineG5),y
            sta PF2

            dey
            beq ScanEnd


ScanBlue    sta WSYNC
                
            lda #BLUE
            sta COLUPF

            lda (LineB0),y
            sta PF0
            lda (LineB1),y
            sta PF1
            lda (LineB2),y
            sta PF2
            lda (LineB3),y
            sta PF0
            lda (LineB4),y
            sta PF1
            lda (LineB5),y
            sta PF2

            dey
            bne ScanRed

ScanEnd

            sty PF0
            sty PF1
            sty PF2

            rts
#endif


    ;--------------------------------------------------------------------------

CalculateDrawPosition

    ; Given the current frame of the player, calculate a draw position such that
    ; the centerpoint is at the creature's (ObjXPosition,ObjYPosition) and that the
    ; left and right edges are then forced to be onscreen.


    ; Calculate the frame definition start address.  Each frame is a 32-byte
    ; definition block giving the bank containing the frame and a matrix of
    ; 16-sprite numbers making up the nx6 grid.

    ; FORMAT:
    ;    BYTE       centerpointx, centerpointy
    ;    BYTES...    0, 1, 2, 3, 4, 5   \
    ;                6, 7, 8, 9,10,11    |  n x 6 matrix of 16-sprite #s
    ;                ....                |
    ;               24,25,26,27,28,29   /

            SETBANK INITBANK

            ldy Player
  
            lda #0
            sta FrameAddress+1

            lda ObjFrame,y              ; animation frame (absolute)
            asl
            adc ObjFrame,y              ; Frametable is bank, AddressLO, AddressHI
            tax

            SETBANK BANK_FRAMETABLE

            lda FrameTable+1,x
            sta FrameAddress
            lda FrameTable+2,x
            sta FrameAddress+1

            lda FrameTable,x
            sta FrameBank
            sta BANK

            ldx #0
            lda ObjFacing,y
            bpl nMirror
            ldx #BIG_ROWS*BIG_COLS
nMirror


    ; Now we have a pointer to the frame definition block (FrameAddress)
    ; Go through the entire BIG_ROWS*BIG_COLS entries and convert the matrix 16-sprite
    ; number into an absolute address pointing to the sprite data.  This will correspond
    ; nicely to the vars used for the actual draw.

            ldy #0
            sty CenterX+1
            lda (FrameAddress),y
            sta CenterX
            iny
            lda (FrameAddress),y
            sta CenterY

            stx Temp+1

Mirrorable  iny

            inc Temp+1
            ldx Temp+1
            lda BaseMirror-1,x
            tax


            tya
            pha

            lda (FrameAddress),y
            asl
            tay
            lda T128,y
            sta SPBASE,x
            lda T128+1,y
            sta SPBASE+1,x

            pla
            tay


            cpy #BIG_ROWS*BIG_COLS+1
            bcc Mirrorable


    ;--------------------------------------------------------------------------
    ; Centerpoint Adjustment based on frame

            ldy Player

            lda ObjFrame,y
            bmi SkipRepos

            lda ObjFacing,y
            bpl nM

            clc
            lda #0
            sbc CenterX
            sta CenterX
            lda #0
            sbc CenterX+1
            sta CenterX+1

            clc
            lda CenterX
            adc #48
            sta CenterX
            lda CenterX+1
            adc #0
            sta CenterX+1

nM

    ; Adjust draw position so that centerpoint is where the ObjXPosition is

            sec
            lda DrawX
            sbc CenterX
            sta VisualX
            lda DrawX+1
            sbc CenterX+1
            sta VisualX+1

            bne OKleft


    ; OK, so the object is offscreen (to the left)
    ; This means that, with the centerpoint adjustment, the left edge of the sprite is offscreen
    ; So we need to adjust the object position (DrawX,DrawY) such that the sprite will be onscreen
    ; As a minimum, we know we want the sprite to be at (0,DrawY).  So we just adjust the
    ; object position by the negative of the offsreen offset

            sec
            lda #0
            sbc VisualX                     ; all we really care about - the offset
            sta Temp
            adc DrawX                       ; adjust (what will be) the actual coordinate
            sta DrawX

    ; and set the visual draw position to left edge

;            lda #1
;            sta VisualX+1
            lda #0
            sta VisualX
            beq Onscreen

OKleft

    ; Check for right-hand side boundary conditions


            sec
            lda VisualX
            sbc #160-48
            sta Temp
            lda VisualX+1
            sbc #1
            bcc Onscreen

            lda Temp
            eor #$FF
            adc #0
            sta Temp

            adc DrawX
            sta DrawX


            lda #160-48
            sta VisualX
;            lda #1
;            sta VisualX+1

Onscreen

            sec
            lda DrawY
            sbc CenterY
            sta VisualY
SkipRepos

    ;--------------------------------------------------------------------------

            ldx Player
            lda DrawX
            sta ObjXPosition,x
            lda DrawY
            sta ObjYPosition,x


            rts


T128
.T128        SET $F000

    REPEAT 22
        .word .T128
.T128        SET .T128 + 128
    REPEND

    ;--------------------------------------------------------------------------


Reset       SETBANK INITBANK
            jmp Cart_Init                   ; init code is in bank 1!


JoystickEvent

        .byte $FF,$FF,$FF,$FF,$FF,$FF,$FF,EVENT_JOY_RIGHT,$FF,$FF,$FF,EVENT_JOY_LEFT,$FF,EVENT_JOY_DOWN,EVENT_JOY_UP,EVENT_JOY_NULL


    ;--------------------------------------------------------------------------
    ; Object X,Y positioning
    ; Timing is absolutely critical here!

PositionObject

    ; Waste scanlines to start of object position, giving our vertical movement

            ldx VisualY
            inx
DelayVert   sta WSYNC
            dex
            bne DelayVert

    ; Now the horizontal magic (which isn't working (1/MAR))

            ldx VisualX
            clc
            lda MoveTable,x
            sta HMP0
            adc #$10
            sta HMP1

            lda #1
            sta VDELP0
            sta VDELP1

            lda #3
            sta NUSIZ0
            sta NUSIZ1

            sta WSYNC

            lda #>JNDelayPos
            sta Temp+1
            lda DelayTable,x
            sta Temp
            jmp (Temp)

            ALIGN   256

JNDelayPos  
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c5

            nop

            sta RESP0
            sta RESP1
            sta WSYNC
            sta HMOVE

            lda #>JNDelayDraw
            sta Temp+1
            lda #<JNDelayDraw
            clc
            adc DelayTable,x
            sta Temp

            lda #0
            sta PF2

            sta WSYNC

            nop
            nop
            nop
            nop
            nop
            nop
            nop
            nop
            nop
            nop

            jmp (Temp)

JNDelayDraw

            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9
            .byte   $c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c9,$c5

            rts


        ALIGN 256
DelayTable
            .byte   37, 37
Q           SET     36
            REPEAT  37
            .byte   Q, Q, Q
Q           SET     Q-1
            REPEND

MoveTable
           .byte   $30, $20
            REPEAT  37
                .byte   $40, $30, $20
            REPEND


;--------------------------------------------------------------------------
; The beautiful, beautiful, draw loop which draws the entire matrix onscreen
; This caters for an adjustable (at compile-time) number of rows and also
; facilitates double-size pixels.

            SUBROUTINE

.S1         SET SPBASE
.S2         SET SPBASE + 2
.S3         SET SPBASE + 4
.S4         SET SPBASE + 6
.S5         SET SPBASE + 8
.S6         SET SPBASE + 10

DrawSprite

RED         = %00110100 ;$42        ; secam 4
GREEN       = %11000110 ;C8 ;$D6              ; secam 8
BLUE        = %01110100             ; secam 2

            jmp (RGBVec)

NextPhase   .byte 1,2,0
PhasePtr    .word .LABR,.LABG,.LABB
            .word .LABPALR,.LABPALG,.LABPALB

            ALIGN 256

.LABG       lda #GREEN          ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABB           ; 3
    
            bmi EndDraw

.LABR       lda #RED            ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABG           ; 3

            bmi EndDraw

.LABB       lda #BLUE           ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABR           ; 3

EndDraw     rts


            ALIGN 256

;PALRED         = $64
;PALGREEN       = $54
;PALBLUE        = $D4

PALRED         = $C8 ;$D6              ; secam 8
PALGREEN       = %00110100 ;$42        ; secam 4
PALBLUE        = %01110100             ; secam 2

.LABPALG    lda #PALGREEN       ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABPALB        ; 3
    
            bmi EndPalDraw

.LABPALR    lda #PALRED         ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABPALG        ; 3

            bmi EndPalDraw

.LABPALB    lda #PALBLUE        ; 2
            sta COLUP0          ; 3
            sta COLUP1          ; 3

            ldy LoopCount       ; 3

            lda (.S1),y         ; 5
            sta GRP0            ; 3
            lda (.S2),y         ; 5
            sta GRP1            ; 3
            lda (.S3),y         ; 5
            sta GRP0            ; 3

            lda (.S6),y         ; 5
            sta Temp            ; 3
            lax (.S5),y         ; 5
            lda (.S4),y         ; 5
            ldy Temp            ; 3
            sta GRP1            ; 3
            stx GRP0            ; 3
            sty GRP1            ; 3
            sta GRP0            ; 3
            
            dec LoopCount       ; 5
            bpl .LABPALR        ; 3

EndPalDraw  rts

 

BaseMirror

.SP         SET 0
            REPEAT BIG_ROWS*BIG_COLS
            .byte .SP
.SP         SET .SP + 2
            REPEND
.SPBASE     SET 0
            REPEAT BIG_ROWS
.SPOFF      SET 10
            REPEAT BIG_COLS
            .byte .SPBASE + .SPOFF
.SPOFF      SET .SPOFF-2
            REPEND
.SPBASE     SET .SPBASE + 12
            REPEND

    ;--------------------------------------------------------------------------

            ORG $7ffa   ;IntVectors

            VECTOR Reset           ; NMI
            VECTOR Reset           ; RESET
            VECTOR Reset           ; IRQ

    		END

